@@ -2,10 +2,11 @@ use std::collections::VecDeque;
 use std::iter::{Chain, Iterator, Once};
 use std::process::ExitStatus;
 use std::rc::Rc;
-use std::slice::Iter;
+use std::collections::vec_deque::Iter;
 use std::sync::Arc;
 
 use futures::future::join_all;
+use futures::join;
 
 use async_recursion::async_recursion;
 use async_trait::async_trait;
@@ -21,13 +22,12 @@ pub struct ScriptGroup<CommandGeneric: Command> {
     pub bail: bool,
     // Enforces that there's always at least 1 script
     pub first: Script<CommandGeneric>,
-    pub rest: Vec<Script<CommandGeneric>>,
+    pub rest: VecDeque<Script<CommandGeneric>>,
 }
 
 impl<CommandGeneric: Command> ScriptGroup<CommandGeneric> {
-    fn iter(&self) -> Chain<Once<&Script<CommandGeneric>>, Iter<Script<CommandGeneric>>> {
-        let test = std::iter::once(&self.first).chain(self.rest.iter());
-        return test;
+    fn iter(&self) -> Chain<Once<&'_ Script<CommandGeneric>>, Iter<'_, Script<CommandGeneric>>> {
+        std::iter::once(&self.first).chain(self.rest.iter())
     }
 }
 
@@ -62,56 +62,50 @@ impl<CommandGeneric: Command> CommandGroup<CommandGeneric> {
 
                 let mut promises = Vec::<_>::new();
 
-                let mut iterator = group.iter();
-                let mut i = iterator.next();
-                let mut next = iterator.next();
+                let mut group_iter = group.iter();
+                let mut command = group_iter.next().unwrap();
 
-                while next.is_some() {
-                    promises.push(i.unwrap().run(parser, VecDeque::new()));
-                    i = next;
-                    next = iterator.next();
+                while let Some(next_command) = group_iter.next() {
+                    promises.push(command.run(parser, VecDeque::new()));
+
+                    command = next_command;
                 }
 
-                promises.push(i.unwrap().run(parser, args));
+                let last_result = command.run(parser, args);
+                let (results, last) = join!(
+                    join_all(promises),
+                    last_result
+                );
 
-                let results = join_all(promises).await;
+                let final_status = results.into_iter().fold(
+                    last.unwrap(),
+                    |prev_exit_status, this_result| {
+                        if let Ok(current_status) = this_result {
+                            return merge_status(prev_exit_status, current_status);
+                        } else {
+                            // TODO: Do something with the error
+                            return prev_exit_status;
+                        }
+                    },
+                );
 
-                let mut status = Option::None;
-                for exit_status_result in results {
-                    let exit_status = exit_status_result.unwrap();
-                    if let Some(current_status) = status {
-                        status = Some(merge_status(current_status, exit_status));
-                    } else {
-                        status = Some(exit_status);
-                    }
-                }
-
-                return Ok(status.unwrap());
+                return Ok(final_status);
             }
-            Self::Series(group) => {
-                let mut iterator = group.iter();
-                let mut i = iterator.next();
-                let mut next = iterator.next();
+            Self::Series(group) => {               
+                let mut rest_iter = group.rest.iter();
+                if let Some(last_command) = rest_iter.next_back() {
+                    let mut exit_status = group.first.run(parser, VecDeque::new()).await.unwrap();
 
-                let mut final_exit_status: Option<ExitStatus> = Option::None;
-
-                while next.is_some() {
-                    let exit_status = i.unwrap().run(parser, VecDeque::new()).await.unwrap();
-                    if let Some(status) = final_exit_status {
-                        final_exit_status = Some(merge_status(exit_status, status));
-                    } else {
-                        final_exit_status = Some(exit_status);
+                    for command in rest_iter {
+                        exit_status = merge_status(exit_status, command.run(parser, VecDeque::new()).await.unwrap());
                     }
-                    i = next;
-                    next = iterator.next();
+
+                    exit_status = merge_status(exit_status, last_command.run(parser, args).await.unwrap());
+
+                    Ok(exit_status)
+                } else {
+                    Ok(group.first.run(parser, args).await.unwrap())
                 }
-
-                final_exit_status = Some(merge_status(
-                    final_exit_status.unwrap(),
-                    i.unwrap().run(parser, args).await.unwrap(),
-                ));
-
-                return Ok(final_exit_status.unwrap());
             }
         }
     }
